########################################################################
# Following values can be modified 
########################################################################
ITERATIONS = 8              # How many javacores you want to produce. DEFAULT is 8.
DELAY = 30                  # Delay in seconds between javacores. DEFAULT is 30.
COLLECTING_OS_DATA = 0      # Collect Windows OS stats. DEFAULT is 0 as false. To turn it on, change to 1.
                            # If it is enabled, C:\temp\windowsperf.RESULTS.<date>.<time> directory will be created.
##################### END MODIFICATION #################################



########################################################################
# USAGE
def usage():
    return """
Usage:

   wsadmin -lang jython -f <path to file>/windowsperf.py server_name1 server_name2

To include node(s), quote the node name with a colon and separate server names with a space.
To add another node, simply add a space between the quoted node names.

   wsadmin -lang jython -f <path to file>/windowsperf.py "node_name:server_name1 server_name2" "node_name2:server_name3"
"""

########################################################################
# Import stuff
########################################################################
import sys
import time
import os
import zipfile
import shutil
import java.lang.Runtime as runtime
import java.io.BufferedReader as bufferedReader
import java.io.InputStreamReader as inputStreamReader

########################################################################
# Defined variables and functions
########################################################################
SCRIPT_VERSION = "2023.06.05"
OUTPUT_FOLDER_NAME = "windowsperf_RESULTS.%s" % time.strftime('%Y%m%d.%H%M%S')
TEMP_DIR = "C:/temp/"
SCRIPT_OUTPUT_DIR = TEMP_DIR + OUTPUT_FOLDER_NAME + "/"
DEFAULT_PARAMETER = "JAVACORE,DELAY="+str(DELAY)
SECTION = "============================================================================"
START_SCRIPT_TIME = time.time()
COLLECTING_CMD_STATS = 0 # ONLY used for debugging. Change to 1 to collect an elapsed time for OS stats command, writing to cmd_stats.txt .

# Write output files to SCRIPT_OUTPUT_DIR folder if COLLECTING_OS_DATA is true.
def write_to_file(file_name,output):
    if COLLECTING_OS_DATA:
        # Split output into lines
        output_lines = output.splitlines()
        # Remove any empty lines from output_lines
        output_lines = [line for line in output_lines if line.strip()]
        # Join the remaining lines back into a string
        clean_output = "\n".join(output_lines)
        outfile = None
        try:
            outfile = open(SCRIPT_OUTPUT_DIR + file_name, "a")
            outfile.write(clean_output.strip())
            outfile.write("\n")
        finally:
            if outfile:
                outfile.close()

# Print a stamp time stamp with message.
# If COLLECTING_OS_DATA is true, write a message to the screen.out.txt file in SCRIPT_OUTPUT_DIR.
def log_stamp(string):
    log = time.strftime('%Y-%m-%d %H:%M:%S') +"\t"+ string
    if COLLECTING_OS_DATA:
        write_to_file("screen.out.txt",log)
    return log
    
def exit_script():
    try:
        #pass
        os._exit(0)
    except SystemExit:
        print("Script exited.")

# Script initialization.
# Creates a SCRIPT_OUTPUT_DIR folder if COLLECTING_OS_DATA is true.
def initialize_script():
    if COLLECTING_OS_DATA:
        try:
            os.makedirs(SCRIPT_OUTPUT_DIR)
            print(log_stamp(SCRIPT_OUTPUT_DIR + " created.") )
        except OSError , e:
            print("Unable writing to %s Reason:\n%s" % (SCRIPT_OUTPUT_DIR,e))
            exit_script()
    print(log_stamp("Script version: %s" % SCRIPT_VERSION))
    print(log_stamp("Iterations: %s" % ITERATIONS))
    print(log_stamp("Delay: %s" % DELAY))
initialize_script()

# Zip and remove SCRIPT_OUTPUT_DIR.
def zip_and_remove_folder():
    print(log_stamp("Preparing for packaging and cleanup..."))
    zip_file = None
    try:
        zip_file = zipfile.ZipFile(TEMP_DIR + OUTPUT_FOLDER_NAME + ".zip", 'w', zipfile.ZIP_DEFLATED)
        for item in os.listdir(SCRIPT_OUTPUT_DIR):
            if os.path.isfile(os.path.join(SCRIPT_OUTPUT_DIR, item)):
                file_path = os.path.abspath(os.path.join(SCRIPT_OUTPUT_DIR, item))
                zip_file.write(file_path, item)
    finally:
        if zip_file:
            zip_file.close()
    shutil.rmtree(SCRIPT_OUTPUT_DIR)
    msg = "Script finished."
    stamp = time.strftime('%Y-%m-%d %H:%M:%S') + "\t"
    print("%s%s" % (stamp,msg))

# Print the end message when the script is done.
def print_end_message():
    zip_file_name = ""
    if COLLECTING_OS_DATA:
        zip_and_remove_folder()
        zip_file_name = "\n\t * "+TEMP_DIR+OUTPUT_FOLDER_NAME+".zip"
    else:
        msg = "Script finished."
        stamp = time.strftime('%Y-%m-%d %H:%M:%S') + "\t"
        print("%s%s" % (stamp,msg))
    print("""
To share with IBM support, zip and upload all the following data for the mentioned servers:%s
\t * WebSphere Application Server logs(systemout.log, native_stderr.log, etc)
\t * javacores
\t * server.xml (profile_root/config/cells/cell_name/nodes/node_name/servers/server_name/
server.xml)
    """ % (zip_file_name))


########################################################################
# Sanity provided inputs
########################################################################
provided_node_server_names = []
def sanity_check_input():
    global provided_node_server_names
    print(log_stamp("Provided servers: %s" % sys.argv))
    # Exit the script for the following reasons:
    # - No provided server names for provided node_name.
    # - No node and/or server names provided.
    if len(sys.argv) > 0 and sys.argv[0]:
        for node_server in sys.argv:
            if ':' in node_server:
                parts = node_server.split(':')
                if len(parts) > 1:
                    if not parts[0]:
                        print(log_stamp("No node name provided in '%s'. Exiting the script." % node_server))
                        print(usage())
                        exit_script()
                    elif not parts[1]:
                        print(log_stamp("No server name provided in '%s'. Exiting the script." % node_server))
                        print(usage())
                        exit_script()
    else:
        print(log_stamp("Node and server names were not provided. Exiting the script."))
        print(usage())
        exit_script()

    provided_node_server_names = sys.argv
    sys.argv = []


# TEST within wsadmin (./wsadmin -jython)
# execfile('path to python file')
"""
sys.argv = [] # clear previous assigned. 
if 0 == len(sys.argv):
    print("Neither node nor app names provided.")
    print(usage())
    sys.argv = raw_input("Enter here: ").split()
    #print("Server names: " + str(sys.argv))
    print('Input provided: %s' % (sys.argv))"""

sanity_check_input()



########################################################################
# Verify running servers
########################################################################
running_servers = []
def verify_running_servers():
    global running_servers
    print(log_stamp("Verifying running servers..."))
    for item in provided_node_server_names:
        full_server_names = []
        if ":" in item:
            node_name, server_names = item.split(":")
            full_server_names = ["node=" + node_name + ",process=" + server_name for server_name in server_names.split()]
        else:
            full_server_names.append("process="+str(item))
        # Exit the script for the folowing reasons:
        # - Server is not running.
        # - There are nodes with the same server name. Provide a speficied node name.
        for full_name in full_server_names:
            output_lines = str(AdminControl.queryNames('WebSphere:type=Server,%s,*' % full_name )).split("\n")
            if len(output_lines) > 1:
                print(log_stamp("There are more than one node with the same server name:%s\n\nPlease specific node name. Exiting the script." % (output_lines)) )
                print(usage())
                exit_script()
            elif not output_lines[0]:
                print(log_stamp("%s is not running. Exiting the script." % full_name))
                exit_script()
        running_servers.extend(full_server_names)
    print(log_stamp("Verified running servers."))
if provided_node_server_names:
    verify_running_servers()


########################################################################
# Send dumps reqeust
########################################################################
parameters = DEFAULT_PARAMETER
for i in range (1, ITERATIONS):
    parameters += ","+str(DEFAULT_PARAMETER)
parametersObj = [parameters]
array_PIDs = []
def send_dump_request():
    print(log_stamp("Sending dumps request..."))
    # Send a dump request to running servers.
    # Get PIDs if COLLECTING_OS_DATA is true and add to array_PIDs.
    for running_server in running_servers:
        runningServerObjectName = AdminControl.completeObjectName('WebSphere:type=Server,'+str(running_server)+',*')
        if COLLECTING_OS_DATA:
            global array_PIDs
            print(log_stamp("Getting PID for %s" % (running_server)))
            array_PIDs.append(int(AdminControl.getAttribute(runningServerObjectName, 'pid')))
        diagPlanMBean = AdminControl.queryNames('WebSphere:type=DiagPlanManager,'+running_server+',*')
        objName = AdminControl.makeObjectName(diagPlanMBean)
        print(log_stamp("Sending a thread dumps request to %s." % (running_server)))
        AdminControl.invoke_jmx(objName, 'setDiagPlan', parametersObj, ["java.lang.String"])
        print(log_stamp("Request submitted to %s" % (running_server)))
    if not COLLECTING_OS_DATA:
        print(log_stamp("Waiting for ~%s seconds until the completion of the task of dump requests..." % (ITERATIONS*DELAY)))
        time.sleep(ITERATIONS*DELAY)
        print_end_message()
if running_servers: 
    send_dump_request()

########################################################################
# Collect OS data if COLLECTING_OS_DATA is true
########################################################################

TOTAL_TIME_SCRIPT_DURATION = ITERATIONS * DELAY

def run_cmd(cmd):
    cmd_runtime = runtime.getRuntime().exec(cmd)
    inputStream = cmd_runtime.getInputStream()
    isr = inputStreamReader(inputStream)
    br = bufferedReader(isr)
    output = ""
    line = br.readLine()
    while line != None:
        output += line + "\n"
        line = br.readLine()
    return output

def run_task_list_cmd():
    start_time = time.time()
    print(log_stamp("Collecting tasklist /v"))
    output = run_cmd('tasklist /v')
    stamp = time.strftime('%Y-%m-%d %H:%M:%S')
    write_to_file("tasklist.out.txt", stamp)
    write_to_file("tasklist.out.txt", output)
    write_to_file("tasklist.out.txt", SECTION)

    # collect stat for cmd
    if COLLECTING_CMD_STATS:
        elapsed_cmd_time = time.time() - start_time
        write_to_file("cmd_stats.txt","%s:%s" % ("tasklist /v",elapsed_cmd_time))


OS_COMMANDS = {
    "memory usage":"wmic os get totalvisiblememorysize,totalvirtualmemorysize,freephysicalmemory,freevirtualmemory",
    "CPU usage":["powershell.exe", "Write-Host \"CPU usage:\" (Get-Counter '\\Processor(_Total)\\% Processor Time').CounterSamples[0].CookedValue'%'"],
    "disk usage":"wmic logicaldisk get size,freespace,caption",
    "paging usage":"wmic PATH Win32_PerfFormattedData_PerfOS_Memory get PageFaultsperSec,PagesPerSec,PageReadsPerSec",
    "netstat -oan": "netstat -oan",
    "netstat -s":"netstat -s"
}

# Collect Windows performance data.
def windowsperf():
    for i in range(ITERATIONS):
        start_collecting_OS_data_time = time.time()
        run_task_list_cmd()
        stamp = time.strftime('%Y-%m-%d %H:%M:%S') + "\t"
        write_to_file("perf.out.txt", stamp)
        for key_cmd_name, command in OS_COMMANDS.items():
            start_cmd_time = time.time()
            print(log_stamp("Collecting %s" % (key_cmd_name)))

            command_output = run_cmd(command)
            write_to_file("perf.out.txt", command_output)
            write_to_file("perf.out.txt","\n")

            # collect stat for cmd
            if COLLECTING_CMD_STATS:
                elapsed_cmd_time = time.time() - start_cmd_time
                write_to_file("cmd_stats.txt","%s:%s" % (key_cmd_name,elapsed_cmd_time))

        write_to_file("perf.out.txt",SECTION)
        threads_cmd = "wmic PATH win32_thread WHERE processhandle=%s GET handle,elapsedtime,usermodetime,kernelmodetime"
        for pid in array_PIDs:
            start_cmd_time = time.time()
            print(log_stamp("Collecting win32_thread for PID %s" % (pid)))
            command = threads_cmd % (pid)
            command_output = run_cmd(command)

            stamp = time.strftime('%Y-%m-%d %H:%M:%S')
            write_to_file(str(pid)+".threads.txt", stamp)
            write_to_file(str(pid)+".threads.txt", command_output)
            write_to_file(str(pid)+".threads.txt", SECTION)

            # collect stat for cmd
            if COLLECTING_CMD_STATS:
                elapsed_cmd_time = time.time() - start_cmd_time
                write_to_file("cmd_stats.txt","%s:%s" % ("wmic PATH win32_thread",elapsed_cmd_time))
        
        OS_data_collection_time = time.time() - start_collecting_OS_data_time        
        
        #If elapsed_time is less than 0 or iteration has reached, break the loop to finish the script.
        elapsed_time = time.time() - START_SCRIPT_TIME
        remaining_time = TOTAL_TIME_SCRIPT_DURATION - elapsed_time
        if remaining_time <= 0 or i == ITERATIONS-1:
            break
        # Skip the DELAY to continue the OS data collection if OS_data_collection_time exceeds the delay. 
        elif (DELAY-OS_data_collection_time <= 0):
            print(log_stamp("Continuing OS collection..."))
            pass
        else:
            print(log_stamp("Pausing data collection for %.0f seconds." % (DELAY-OS_data_collection_time)))
            time.sleep(DELAY-OS_data_collection_time)
    print_end_message()

if COLLECTING_OS_DATA and running_servers:
    windowsperf()

